/*
 * Copyright (C) 2013 DENSO CORPORATION
 * Copyright (c) 2013 BMW Car IT GmbH
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef IVI_HMI_CONTROLLER_CLIENT_PROTOCOL_H
#define IVI_HMI_CONTROLLER_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct ivi_hmi_controller;
struct wl_seat;

extern const struct wl_interface ivi_hmi_controller_interface;

#ifndef IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM
#define IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM
enum ivi_hmi_controller_layout_mode {
	IVI_HMI_CONTROLLER_LAYOUT_MODE_TILING = 0,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_SIDE_BY_SIDE = 1,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_FULL_SCREEN = 2,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_RANDOM = 3,
};
#endif /* IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM */

#ifndef IVI_HMI_CONTROLLER_HOME_ENUM
#define IVI_HMI_CONTROLLER_HOME_ENUM
enum ivi_hmi_controller_home {
	IVI_HMI_CONTROLLER_HOME_OFF = 0,
	IVI_HMI_CONTROLLER_HOME_ON = 1,
};
#endif /* IVI_HMI_CONTROLLER_HOME_ENUM */

/**
 * ivi_hmi_controller - set up and control IVI style UI
 * @workspace_end_control: notify controlling workspace end
 *
 * 
 */
struct ivi_hmi_controller_listener {
	/**
	 * workspace_end_control - notify controlling workspace end
	 * @is_controlled: (none)
	 *
	 * 
	 */
	void (*workspace_end_control)(void *data,
				      struct ivi_hmi_controller *ivi_hmi_controller,
				      int32_t is_controlled);
};

static inline int
ivi_hmi_controller_add_listener(struct ivi_hmi_controller *ivi_hmi_controller,
				const struct ivi_hmi_controller_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) ivi_hmi_controller,
				     (void (**)(void)) listener, data);
}

#define IVI_HMI_CONTROLLER_UI_READY	0
#define IVI_HMI_CONTROLLER_WORKSPACE_CONTROL	1
#define IVI_HMI_CONTROLLER_SWITCH_MODE	2
#define IVI_HMI_CONTROLLER_HOME	3

#define IVI_HMI_CONTROLLER_UI_READY_SINCE_VERSION	1
#define IVI_HMI_CONTROLLER_WORKSPACE_CONTROL_SINCE_VERSION	1
#define IVI_HMI_CONTROLLER_SWITCH_MODE_SINCE_VERSION	1
#define IVI_HMI_CONTROLLER_HOME_SINCE_VERSION	1

static inline void
ivi_hmi_controller_set_user_data(struct ivi_hmi_controller *ivi_hmi_controller, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) ivi_hmi_controller, user_data);
}

static inline void *
ivi_hmi_controller_get_user_data(struct ivi_hmi_controller *ivi_hmi_controller)
{
	return wl_proxy_get_user_data((struct wl_proxy *) ivi_hmi_controller);
}

static inline uint32_t
ivi_hmi_controller_get_version(struct ivi_hmi_controller *ivi_hmi_controller)
{
	return wl_proxy_get_version((struct wl_proxy *) ivi_hmi_controller);
}

static inline void
ivi_hmi_controller_destroy(struct ivi_hmi_controller *ivi_hmi_controller)
{
	wl_proxy_destroy((struct wl_proxy *) ivi_hmi_controller);
}

static inline void
ivi_hmi_controller_UI_ready(struct ivi_hmi_controller *ivi_hmi_controller)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_UI_READY);
}

static inline void
ivi_hmi_controller_workspace_control(struct ivi_hmi_controller *ivi_hmi_controller, struct wl_seat *seat, uint32_t serial)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_WORKSPACE_CONTROL, seat, serial);
}

static inline void
ivi_hmi_controller_switch_mode(struct ivi_hmi_controller *ivi_hmi_controller, uint32_t layout_mode)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_SWITCH_MODE, layout_mode);
}

static inline void
ivi_hmi_controller_home(struct ivi_hmi_controller *ivi_hmi_controller, uint32_t home)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_HOME, home);
}

#ifdef  __cplusplus
}
#endif

#endif
